/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter.contentnegotiation.parser;

/**
 * The part element of the negotiation XML
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public class HeaderPart
{

    private static final int HASH_SEED = 7;
    private static final int HASH_MULT = 19;

    protected String name;
    protected String value;

    public HeaderPart()
    {
    }
    
    /**
     * Get the name of the header element
     * @return The name of the header element
     * @since 0.0.13
     */
    public String getName()
    {
        return name;
    }

    /**
     * Set the name of the header element
     * @param name The name of the header element
     * @since 0.0.13
     */
    public void setName(final String name)
    {
        this.name = name;
    }

    /**
     * Get the value of the header element
     * @return The value of the header element
     * @since 0.0.13
     */
    public String getValue()
    {
        return value;
    }

    /**
     * Set the value of the header element
     * @param value The value of the header element
     * @since 0.0.13
     */
    public void setValue(final String value)
    {
        this.value = value;
    }

    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        if (name != null)
        {
            hash = (hash * HASH_MULT) + name.hashCode();
        }
        else
        {
            hash = (hash * HASH_MULT) + 3;
        }
        if (value != null)
        {
            hash = (hash * HASH_MULT) + value.hashCode();
        }
        else
        {
            hash = (hash * HASH_MULT) + 7;
        }
        return hash;
    }

    @Override
    public boolean equals(final Object o)
    {
        if (o == null)
        {
            return false;
        }
        else if (o instanceof HeaderPart)
        {
            HeaderPart co = (HeaderPart)o;
            boolean allEqual = true;
            if (this.name != null)
            {
                if (co.name != null)
                {
                    allEqual = allEqual && this.name.equals(co.name);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.name != null)
            {
                allEqual = false;
            }
            if (this.value != null)
            {
                if (co.value != null)
                {
                    allEqual = allEqual && this.value.equals(co.value);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.value != null)
            {
                allEqual = false;
            }
            return allEqual;
        }
        else
        {
            return false;
        }
    }

}
