/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter.contentnegotiation.parser;

/**
 * The output element of the negotiation XML.
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public class Output
{

    private static final int HASH_SEED = 1;
    private static final int HASH_MULT = 7;

    protected String forward;
    protected Header header;
    protected Redirect redirect;
    protected Boolean fail;

    public Output()
    {
    }
    
    /**
     * Get the forward of the output.
     * @return The forward of the output
     * @since 0.0.13
     */
    public String getForward()
    {
        return this.forward;
    }

    /**
     * Set the forward of the output.
     * @param value The forward of the output
     * @since 0.0.13
     */
    public void setForward(String value)
    {
        this.forward = value;
    }

    /**
     * Get the header of the output.
     * @return The header of the output
     * @since 0.0.13
     */
    public Header getHeader()
    {
        return header;
    }

    /**
     * Set the header of the output.
     * @param value The header of the output
     * @since 0.0.13
     */
    public void setHeader(Header value)
    {
        this.header = value;
    }

    /**
     * Get the redirect of the output.
     * @return The redirect of the output
     * @since 0.0.13
     */
    public Redirect getRedirect()
    {
        return redirect;
    }

    /**
     * Set the redirect of the output.
     * @param value The redirect of the output
     * @since 0.0.13
     */
    public void setRedirect(Redirect value)
    {
        this.redirect = value;
    }

    /**
     * Get the fail of the output.
     * @return The fail of the output
     * @since 0.0.13
     */
    public Boolean isFail()
    {
        return fail;
    }

    /**
     * Set the fail of the output.
     * @param value The fail of the output
     * @since 0.0.13
     */
    public void setFail(Boolean value)
    {
        this.fail = value;
    }

    @Override
    public int hashCode()
    {
        int hash = HASH_SEED;
        hash = (hash * HASH_MULT);
        if (forward != null)
        {
            hash += forward.hashCode();
        }
        else
        {
            hash += 1;
        }
        hash = (hash * HASH_MULT);
        if (header != null)
        {
            hash += header.hashCode();
        }
        else
        {
            hash += 2;
        }
        hash = (hash * HASH_MULT);
        if (redirect != null)
        {
            hash += redirect.hashCode();
        }
        else
        {
            hash += 3;
        }
        hash = (hash * HASH_MULT);
        if (fail != null)
        {
            hash += fail.hashCode();
        }
        else
        {
            hash += 5;
        }
        return hash;
    }

    @Override
    public boolean equals(Object o)
    {
        if (o == null)
        {
            return false;
        }
        else if (o instanceof Output)
        {
            Output co = (Output)o;
            boolean allEqual = true;
            if (this.forward != null)
            {
                if (co.forward != null)
                {
                    allEqual = allEqual && this.forward.equals(co.forward);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.forward != null)
            {
                allEqual = false;
            }
            if (this.header != null)
            {
                if (co.header != null)
                {
                    allEqual = allEqual && this.header.equals(co.header);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.header != null)
            {
                allEqual = false;
            }
            if (this.redirect != null)
            {
                if (co.redirect != null)
                {
                    allEqual = allEqual && this.redirect.equals(co.redirect);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.redirect != null)
            {
                allEqual = false;
            }
            if (this.fail != null)
            {
                if (co.fail != null)
                {
                    allEqual = allEqual && this.fail.equals(co.fail);
                }
                else
                {
                    allEqual = false;
                }
            }
            else if (co.fail != null)
            {
                allEqual = false;
            }
            return allEqual;
        }
        else
        {
            return false;
        }
    }

}
