/* Copyright © 2013 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
 * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter.contentnegotiation.variantsource;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.mattunderscore.filter.contentnegotiation.parser.Header;
import com.mattunderscore.filter.contentnegotiation.parser.HeaderPart;
import com.mattunderscore.filter.contentnegotiation.parser.Output;
import com.mattunderscore.filter.contentnegotiation.parser.Redirect;
import com.mattunderscore.filter.contentnegotiation.parser.Variant;

/**
 * Parser for the Negotiation XML file. It is not intended to be called directly. This parser is
 * called by {@link FilterVariantXMLSource} and that should be used to get variants from an XML file.
 * 
 * @author Matt Champion
 * @since 0.2.0
 */
class VariantXMLParser extends VariantSAXHandler
{
    private Stack<StringBuilder> elementCharacterData = new Stack<StringBuilder>();
    private List<Variant> variants;
    private Variant currentVariant;
    private Output currentOutput;
    private Header currentHeader;
    private Redirect currentRedirect;
    private HeaderPart currentHeaderPart;
    private StringBuilder tempString;
    private List<HeaderPart> currentHeaderParts;

    @Override
    public void startElement(String s, String s1, String elementName, Attributes attributes)
            throws SAXException
    {
        tempString = new StringBuilder(50);
        elementCharacterData.push(tempString);
        if (elementName.equals("variants"))
        {
            variants = new ArrayList<Variant>();
        }
        else if (elementName.equals("variant"))
        {
            currentVariant = new Variant();
        }
        else if (elementName.equals("output"))
        {
            currentOutput = new Output();
        }
        else if (elementName.equals("header"))
        {
            currentHeader = new Header();
            currentHeaderParts = new ArrayList<HeaderPart>();
        }
        else if (elementName.equals("redirect"))
        {
            currentRedirect = new Redirect();
        }
        else if (elementName.equals("fail"))
        {
            currentOutput.setFail(true);
        }
        else if (elementName.equals("part"))
        {
            currentHeaderPart = new HeaderPart();
        }
    }

    @Override
    public void endElement(String prefix, String localName, String qualifiedName)
            throws SAXException
    {
        if (qualifiedName.equals("variant"))
        {
            variants.add(currentVariant);
        }
        else if (qualifiedName.equals("urlPattern"))
        {
            currentVariant.setUrlPattern(tempString.toString());
        }
        else if (qualifiedName.equals("content-type"))
        {
            currentVariant.setContentType(tempString.toString());
        }
        else if (qualifiedName.equals("language"))
        {
            currentVariant.setLanguage(tempString.toString());
        }
        else if (qualifiedName.equals("output"))
        {
            currentVariant.setOutput(currentOutput);
        }
        else if (qualifiedName.equals("forward"))
        {
            currentOutput.setForward(tempString.toString());
        }
        else if (qualifiedName.equals("redirect"))
        {
            currentOutput.setRedirect(currentRedirect);
        }
        else if (qualifiedName.equals("header"))
        {
            currentHeader.setPart(currentHeaderParts);
            currentOutput.setHeader(currentHeader);
        }
        else if (qualifiedName.equals("fail"))
        {
            currentOutput.setFail(true);
        }
        else if (qualifiedName.equals("part"))
        {
            currentHeaderParts.add(currentHeaderPart);
        }
        else if (qualifiedName.equals("code"))
        {
            int code = Integer.parseInt(tempString.toString());
            if (currentRedirect != null)
            {
                currentRedirect.setCode(code);
            }
            else if (currentHeader != null)
            {
                currentHeader.setCode(code);
            }
        }
        else if (qualifiedName.equals("location"))
        {
            currentRedirect.setLocation(tempString.toString());
        }
        else if (qualifiedName.equals("name"))
        {
            currentHeaderPart.setName(tempString.toString());
        }
        else if (qualifiedName.equals("value"))
        {
            currentHeaderPart.setValue(tempString.toString());
        }
        elementCharacterData.pop();
        if (!elementCharacterData.isEmpty())
        {
            tempString = elementCharacterData.peek();
        }
    }

    @Override
    public void characters(char[] chars, int start, int length)
    {
        tempString.append(new String(chars,start,length));
    }

    @Override
    public List<Variant> getVariants()
    {
        return variants;
    }
    
    @Override
    public void error(SAXParseException exception) throws SAXParseException {
        throw exception;
    }
}
